package it.amattioli.dominate.sessions;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * This {@link SessionManager} implementation is able to delegate its behavior
 * to other session managers so sessions of various types can be retrieved.
 * 
 * @author andrea
 *
 */
public class CompositeSessionManager implements SessionManager {
	private static final Logger logger = LoggerFactory.getLogger(CompositeSessionManager.class);
	private static final String INIT_PROPERTIES = "CompositeSessionManagerInit.properties";
	private static Map<Class<?>, Class<? extends SessionManager>> managersDefinition = new HashMap<Class<?>, Class<? extends SessionManager>>();
	static {
		Properties initProps = new Properties();
		try {
			InputStream initStream = CompositeSessionManager.class.getClassLoader().getResourceAsStream(INIT_PROPERTIES);
			if (initStream == null) {
				initStream = CompositeSessionManager.class.getResourceAsStream(INIT_PROPERTIES);
			}
			initProps.load(initStream);
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		for (Object key: initProps.keySet()) {
			try {
				Class<?> sessionClass = Class.forName((String)key);
				Class<? extends SessionManager> managerClass = (Class<? extends SessionManager>)Class.forName(initProps.getProperty((String)key));
				registerManager(sessionClass, managerClass);
			} catch (ClassNotFoundException e) {
				throw new RuntimeException(e);
			}
		}
	}
	
	public static void registerManager(Class<?> sessionClass, Class<? extends SessionManager> managerClass) {
		managersDefinition.put(sessionClass, managerClass);
	}
	
	private SessionMode mode;
	private Map<Class<?>, SessionManager> managers = new HashMap<Class<?>, SessionManager>();
	
	public CompositeSessionManager(SessionMode mode) {
		this.mode = mode;
		for (Class<?> sessionClass: managersDefinition.keySet()) {
			managers.put(sessionClass, createManager(managersDefinition.get(sessionClass), mode));
		}
	}
	
	private SessionManager createManager(Class<? extends SessionManager> managerClass, SessionMode mode) {
		try {
			Constructor<? extends SessionManager> c = managerClass.getConstructor(SessionMode.class);
			return c.newInstance(mode);
		} catch (SecurityException e) {
			throw new RuntimeException(e);
		} catch (NoSuchMethodException e) {
			throw new RuntimeException(e);
		} catch (IllegalArgumentException e) {
			throw new RuntimeException(e);
		} catch (InstantiationException e) {
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		} catch (InvocationTargetException e) {
			throw new RuntimeException(e);
		}
	}

	@Override
	public <S> S getSession(Class<S> sessionClass) {
		return managers.get(sessionClass).getSession(sessionClass);
	}

	@Override
	public SessionMode getSessionMode() {
		return mode;
	}

	@Override
	public boolean hasSessionMode(SessionMode sMode) {
		return this.mode.equals(sMode);
	}

	@Override
	public void release() {
		for (SessionManager curr: managers.values()) {
			curr.release();
		}
	}
	
	@Override
	public boolean isReleased() {
		return managers.values().iterator().next().isReleased();
	}

	@Override
	public void reset() {
		for (SessionManager curr: managers.values()) {
			curr.reset();
		}
	}

}
